-- See all the customers with Autopayments
DROP TABLE IF EXISTS onboarding.autopayments_npath;
CREATE TABLE onboarding.autopayments_npath
    DISTRIBUTE BY HASH(cmusno) 
    COMPRESS AS (
    SELECT *
    FROM NPATH(
    ON (SELECT *
    FROM onboarding.event_list_18_0_to_90_removed
    WHERE domain_type IN ('acq_chn', 'AUTO_PAYMENT') AND event_desc  NOT IN ('AutoPmtOpen_SIGORTA', 
    'AutoPmtOpen_OTHERS', 'AutoPmtCancel_SIGORTA', 'AutoPmtCancel_OTHERS')
    )
    PARTITION BY cmusno
    ORDER BY row_number
    MODE('NONOVERLAPPING')
    SYMBOLS(domain_type IN ('acq_chn') AS a,
    domain_type IN ('AUTO_PAYMENT') AS b,
    domain_type NOT IN ('AUTO_PAYMENT') AS c,
    true as d
    )
    PATTERN('a.b+')  -- Get all the customers where there is one autopayment event
    RESULT(ACCUMULATE(event_desc OF ANY(a, b)) AS path,
    FIRST(cmusno OF a) AS cmusno,
    COUNT(cmusno OF ANY(a, b)) AS lenPath,
    FIRST(event_date OF a) AS start_date,
    LAST(event_date OF ANY(b)) AS end_date)
    )
    );
GRANT ALL ON onboarding.autopayments_npath TO db_analytics;

--Aggregate paths, calculate average duration for events and count the paths
DROP TABLE IF EXISTS onboarding.freq_autopayments_npath;
CREATE TABLE onboarding.freq_autopayments_npath
    DISTRIBUTE BY HASH(path)
    COMPRESS AS (
    SELECT path, CAST(avg( CAST (end_date AS DATE) - CAST (start_date AS DATE) ) AS INTEGER) duration, lenPath, count(*) cnt
    FROM onboarding.autopayments_npath
    WHERE lenPath >= 2
    GROUP BY 1, lenPath
    );
GRANT ALL ON onboarding.freq_autopayments_npath TO db_analytics;

-- All Paths count
SELECT count (*)
    FROM onboarding.autopayments_npath;   --3614 customers go for Autopayment

SELECT count (DISTINCT cmusno)
    FROM onboarding.autopayments_npath; --3614

-- See the Unique Paths
SELECT count (*)
    FROM onboarding.freq_autopayments_npath; -- 330 Unique Paths

-- Single user paths
SELECT count(*)
    FROM onboarding.freq_autopayments_npath
    WHERE cnt = 1; --205 individual paths

-- Customers having common paths equal to 5 or more
SELECT sum(cnt)
    FROM onboarding.freq_autopayments_npath
    WHERE cnt >= 5; --3245

--Customers where path length max path length is 8
SELECT sum(cnt)
    FROM onboarding.freq_autopayments_npath
    WHERE lenpath <= 8; --3502
    
-- See the customers where we have multipe autopayment events
SELECT *
    FROM onboarding.autopayments_npath
    WHERE path like '%' || '[0_SUBE, AutoPmtOpen_OFO, AutoPmtOpen_OFO, AutoPmtOpen_OFO, AutoPmtOpen_OFO, AutoPmtOpen_OFO, AutoPmtOpen_OFO, AutoPmtOpen_OFO' || '%';
    
-- Visualization Code for AppCenter
--Tree visualisation code
--INSERT INTO app_center_visualizations  (json)
--SELECT json FROM Visualizer (
--    ON (select * from onboarding.freq_autopayments_npath) PARTITION BY 1 
--AsterFunction('npath')
--Title('acquisationchannel')
--VizType('tree')   
--);

--Sankey visualization Code
--INSERT INTO app_center_visualizations  (json) 
--SELECT json FROM Visualizer (
--   ON (select * from onboarding.freq_autopayments_npath where lenpath <= 8) PARTITION BY 1 
--AsterFunction('npath') 
--Title('acquisationchannel') 
--VizType('sankey')
--);