-- See all the customers who are going for 2nd application after the first application approval
-- Only keeping the events from application domain. It is easier to visualize paths for specific events only
DROP TABLE IF EXISTS onboarding.openings_npath;
CREATE TABLE onboarding.openings_npath
    DISTRIBUTE BY HASH(cmusno) 
    COMPRESS AS (
    SELECT *
    FROM NPATH(
    ON (SELECT *
    FROM onboarding.event_list_18_0_to_90_removed
    WHERE event_desc in ('CCOpen_SUP-OTHR'
,'CCOpen_PRM-PLT'
,'CCOpen_PRM-CTEB'
,'CCOpen_SUP-PLT'
,'CCOpen_PRM-OTHR'
,'DCOpen'
,'DepositOpening_O/N'
,'DepositOpening_DemandDeposit'
,'DepositOpening_TimeDeposit'
,'MtgeOpenTL'
,'CarLnOpenYP'
,'PerLnOpenTL'
,'MtgeOpenYP'
,'WorkplaceLnOpenTL'
,'CarLnOpenTL'
,'PerFinLnOpenTL'
,'PerLnOpenYP'
,'ConnectionErr_KMDAccOpening'
,'ConnectionErr_AccOpening'
,'ConnectionErr_OtherAccOpening'
,'ConnectionErr_CeptetebAccOpening'
,'AutoPmtOpen_BES',
'AutoPmtOpen_OFO',
'AutoPmtOpen_OTHERS') or domain_type='acq_chn'
		) 
    PARTITION BY cmusno
    ORDER BY row_number
    MODE('NONOVERLAPPING')
    SYMBOLS(
	domain_type IN ('acq_chn') AS b,
	true as a
    )
    --PATTERN('b.a+')  -- Get all the customers where there is one application approval and customers are going for another application
	PATTERN('b.a{1,4}')
    RESULT(ACCUMULATE(event_desc OF ANY(b,a)) AS path,
    FIRST(cmusno OF b) AS cmusno,
    COUNT(cmusno OF ANY(b,a)) AS lenPath,
    FIRST(event_date OF a) AS start_date,
    LAST(event_date OF ANY(a)) AS end_date)
    )
    );
GRANT ALL ON onboarding.openings_npath TO db_analytics;

select distinct event_desc, domain_type from onboarding.event_list_18_0_to_90_removed;

--Aggregate paths, calculate average duration for events and count the paths
DROP TABLE IF EXISTS onboarding.freq_openings_npath;
CREATE TABLE onboarding.freq_openings_npath
    DISTRIBUTE BY HASH(path)
    COMPRESS AS (
    SELECT path, CAST(avg( CAST (end_date AS DATE) - CAST (start_date AS DATE) ) AS INTEGER) duration, lenPath, count(*) cnt
    FROM onboarding.openings_npath
    WHERE lenPath >= 2
    GROUP BY 1, lenPath
    );
GRANT ALL ON onboarding.freq_openings_npath TO db_analytics;



-- All Paths count
SELECT count (*)
    FROM onboarding.openings_npath;   --32951 customers go for 2nd application

SELECT count (DISTINCT cmusno)
    FROM onboarding.openings_npath; --32951

-- See the Unique Paths
SELECT count (*)
    FROM onboarding.freq_openings_npath; -- 3508 Unique Paths

-- Single user paths
SELECT count(*)
    FROM onboarding.freq_openings_npath
    WHERE cnt = 1; --2487 individual paths

-- Customers having common paths equal to 5 or more
SELECT sum(cnt)
    FROM onboarding.freq_openings_npath
    WHERE cnt >= 5; --28974

--Customers where path length max path length is 8
SELECT sum(cnt)
    FROM onboarding.freq_openings_npath
    WHERE lenpath <= 8; --29347

SELECT *
    FROM onboarding.freq_openings_npath
    WHERE cnt >=2;

-- Visualization Code for AppCenter
--INSERT INTO app_center_visualizations  (json) 
--SELECT json FROM Visualizer (
--ON (select * from onboarding.freq_openings_npath where cnt>=3   )  PARTITION BY 1 
--AsterFunction('npath') 
--Title('40-openings') 
--VizType('tree')
--);


--INSERT INTO app_center_visualizations  (json) 
--SELECT json FROM Visualizer (
--    ON (select * from onboarding.freq_openings_npath) PARTITION BY 1 
--AsterFunction('npath') 
--Title('40-openings') 
--VizType('sankey'));